@@ -16,6 +16,7 @@ from typing import Protocol
 import functools
 import ujson as json
 import pickle
+from timeit import default_timer
 from .utils import logger
 
 get_var = os.getenv
@@ -355,8 +356,61 @@ def get_config():
     }
 
 
+__s3__: Optional["S3"] = None
+
+
 class S3(object):
-    """S3 Wrapper"""
+    MAX_SESSION_LENGTH = 60 * 60 * 11
+
+    def __init__(self):
+        self.config = get_config()
+
+        # check if init of this class was already called once
+        has_session = hasattr(self, "start") and hasattr(self, "s3")
+
+        if has_session and not self._session_exired():
+            # logger.debug("reusing active session")
+            pass
+        else:
+            self.invalidate_session()
+
+    def invalidate_session(self):
+        """ invalidates current session and recreates necessary properties of the class"""
+
+        args = {}
+
+        if self.config['profile'] == "":
+            access_key_id = self.config['aws_access_key_id']
+            logger.debug(f"start s3 session using access key {access_key_id}")
+            session = boto3
+            args = dict(
+                region_name=self.config['s3_region'],
+                aws_access_key_id=access_key_id,
+                aws_secret_access_key=self.config['aws_secret_access_key']
+            )
+        else:
+            profile = self.config['profile']
+            logger.debug(f"start s3 session using profile {profile}")
+            session = boto3.session.Session(profile_name=profile)
+
+        self.s3_client = session.client('s3', **args)
+        self.s3 = session.resource('s3', **args)
+        self.transfer = boto3.s3.transfer.S3Transfer(client=self.s3_client)
+
+        # set the start of the new session
+        self.start: float = default_timer()
+
+    def _session_exired(self):
+        total_seconds = default_timer() - self.start
+        return total_seconds > self.MAX_SESSION_LENGTH
+
+    def __new__(cls, *args, **kwargs):
+        global __s3__
+
+        if __s3__ is None or __s3__._session_exired():
+            __s3__ = super(S3, cls).__new__(cls)
+
+        return __s3__
 
     @staticmethod
     def split_uri(uri: str) -> Tuple[str, str]:
